@@ -11,11 +11,20 @@ You should have received a copy of the license along with this work.
 If not, see <https://creativecommons.org/licenses/by-sa/4.0/>.
 """
 
+from cwl_loader.utils import (
+    assert_process_contained,
+    to_dict
+)
 from cwl_utils.parser import Process
+from datetime import datetime
 from enum import (
     auto,
     Enum
 )
+from importlib.metadata import (
+    version,
+    PackageNotFoundError
+)
 from jinja2 import (
     Environment,
     PackageLoader
@@ -29,17 +38,23 @@ from typing import (
     get_origin
 )
 
+import time
+
 class DiagramType(Enum):
     '''The supported PlantUML diagram types'''
-    COMPONENTS = auto()
+    ACTIVITY = auto()
+    '''Represents the PlantUML `activity' diagram'''
+    COMPONENT = auto()
     '''Represents the PlantUML `components' diagram'''
     CLASS = auto()
     '''Represents the PlantUML `class' diagram'''
     SEQUENCE = auto()
     '''Represents the PlantUML `sequence' diagram'''
+    STATE = auto()
+    '''Represents the PlantUML `state' diagram'''
 
 def _to_puml_name(identifier: str) -> str:
-    return identifier.replace('-', '_')
+    return identifier.replace('-', '_').replace('/', '_')
 
 def _type_to_string(typ: Any) -> str:
     if get_origin(typ) is Union:
@@ -56,6 +71,12 @@ def _type_to_string(typ: Any) -> str:
 
     return typ.__name__
 
+def _get_version() -> str:
+    try:
+        return version("cwl2puml")
+    except PackageNotFoundError:
+        return 'N/A'
+
 _jinja_environment = Environment(
     loader=PackageLoader(
         package_name='cwl2puml'
@@ -67,7 +88,8 @@ _jinja_environment.filters['type_to_string'] = _type_to_string
 def to_puml(
     cwl_document: Process | List[Process],
     diagram_type: DiagramType,
-    output_stream: TextIO
+    output_stream: TextIO,
+    workflow_id: str = 'main'
 ):
     '''
     Converts a CWL,m given its document model, to a PlantUML diagram.
@@ -80,8 +102,21 @@ def to_puml(
     Returns:
         `None`: none
     '''
+    assert_process_contained(
+        process=cwl_document,
+        process_id=workflow_id
+    )
+    
+    index = to_dict(cwl_document) if isinstance(cwl_document, list) else { workflow_id: cwl_document }
+
     template = _jinja_environment.get_template(f"{diagram_type.name.lower()}.puml")
 
-    workflows = cwl_document if isinstance(cwl_document, list) else [cwl_document]
-
-    output_stream.write(template.render(workflows=workflows))
+    output_stream.write(
+        template.render(
+            version=_get_version(),
+            timestamp=datetime.fromtimestamp(time.time()).isoformat(timespec='milliseconds'),
+            workflows=index.values(),
+            workflow_id=workflow_id,
+            index=index
+        )
+    )
